-------------------
--SUPPORT SCRIPTS--
-------------------
--load filepath helper
require("scripts/filepathhelper")
dofile("scripts/filepathhelper")

--load other support scripts
dofile("scripts/callbackmerger")
dofile("scripts/customcallbacks")
dofile("scripts/savehelper")


----------------
--REGISTER MOD--
----------------
local mod = RegisterMod("pee board", 1)


--------------------
--ENUM DEFINITIONS--
--------------------
PlayerType.PLAYER_PIBBLES = Isaac.GetPlayerTypeByName("Pibbles")
PlayerType.PLAYER_ONYX = Isaac.GetPlayerTypeByName("Onyx")

NullItemID.ID_PIBBLES = Isaac.GetCostumeIdByPath("gfx/characters/peeboard/character_pibbles.anm2") --IF ONLY WE COULD ASSOCIATE COSTUMES WITH PLAYERS IN THE XML... HINT HINT NICALIS
NullItemID.ID_PIBBLES_GHOST = Isaac.GetCostumeIdByPath("gfx/characters/peeboard/character_pibbles_ghost.anm2")
NullItemID.ID_ONYX = Isaac.GetCostumeIdByPath("gfx/characters/peeboard/character_onyx.anm2")
NullItemID.ID_ONYX_GHOST = Isaac.GetCostumeIdByPath("gfx/characters/peeboard/character_onyx_ghost.anm2")
NullItemID.ID_LYCANTHROPY = Isaac.GetCostumeIdByPath("gfx/characters/peeboard/item_lycanthropy.anm2")
NullItemID.ID_GUPPY_PIBBLESONYX = Isaac.GetCostumeIdByPath("gfx/characters/peeboard/transformation_guppy.anm2")

CollectibleType.COLLECTIBLE_PEE_BOARD = Isaac.GetItemIdByName("Pee Board")
CollectibleType.COLLECTIBLE_FEMBOY = Isaac.GetItemIdByName("Femboy")
CollectibleType.COLLECTIBLE_LYCANTHROPY = Isaac.GetItemIdByName("Lycanthropy")


---------------------
--SAVE HELPER SETUP--
---------------------
SaveHelper.AddMod(mod)
SaveHelper.DefaultLevelSave(mod, {
	LycanthropyActive = {false,false,false,false}
})


---------
--CACHE--
---------
local vecZero = Vector(0,0)


---------------------
--GENERAL FUNCTIONS--
---------------------
--these are used within the callback functions

--adds the costume and sets IsDog or IsWolf to true if they are pibbles or onyx and not already a dog or wolf
--used when the player starts the game as pibbles/onyx or transforms into one of them somehow, evn though theres literally no mod api function to let mods turn you into characters. yes i know clicker exists. no it doesn't turn you into mod characters. no, just because we have clicker that doesn't mean that makes it ok that we dont have a function to change the character. shut up.
function mod:TryInitCharacter(player)

	local data = player:GetData()
	
	local isGhost = false
	if data.TrueCoop and data.TrueCoop.Save and data.TrueCoop.Save.IsGhost then
		isGhost = true
	end
	
	if not isGhost then
	
		local playerType = player:GetPlayerType()
	
		if not data.IsDog and playerType == PlayerType.PLAYER_PIBBLES then
		
			data.IsDog = true
			player:AddNullCostume(NullItemID.ID_PIBBLES)
			
			--try to fix pibbles' spritesheet when reloading the game
			if data.TrueCoop and data.TrueCoop.JustCalledUpdate then
			
				local sprite = player:GetSprite()
				sprite:ReplaceSpritesheet(1, "gfx/characters/peeboard/costumes/character_pibbles.png")
				sprite:ReplaceSpritesheet(4, "gfx/characters/peeboard/costumes/character_pibbles.png")
				sprite:ReplaceSpritesheet(12, "gfx/characters/peeboard/costumes/character_pibbles.png")
				sprite:LoadGraphics()
				
			end
			
			--pibbles shouldnt have dog tooth's costume
			local itemConfig = Isaac.GetItemConfig()
			local dogToothConfig = itemConfig:GetCollectible(CollectibleType.COLLECTIBLE_DOG_TOOTH)
			player:RemoveCostume(dogToothConfig)
			
			player:AddCacheFlags(CacheFlag.CACHE_FIREDELAY)
			player:AddCacheFlags(CacheFlag.CACHE_DAMAGE)
			player:EvaluateItems()
		
		end
		
		if not data.IsWolf and playerType == PlayerType.PLAYER_ONYX then
		
			data.IsWolf = true
			player:AddNullCostume(NullItemID.ID_ONYX)
			
			--try to fix onyx's spritesheet when reloading the game
			if data.TrueCoop and data.TrueCoop.JustCalledUpdate then
			
				local sprite = player:GetSprite()
				sprite:ReplaceSpritesheet(1, "gfx/characters/peeboard/costumes/character_onyx.png")
				sprite:ReplaceSpritesheet(4, "gfx/characters/peeboard/costumes/character_onyx.png")
				sprite:ReplaceSpritesheet(12, "gfx/characters/peeboard/costumes/character_onyx.png")
				sprite:LoadGraphics()
				
			end
			
			--onyx shouldnt have dog tooth's or mom's contacts' costumes
			local itemConfig = Isaac.GetItemConfig()
			local dogToothConfig = itemConfig:GetCollectible(CollectibleType.COLLECTIBLE_DOG_TOOTH)
			local momsContactsConfig = itemConfig:GetCollectible(CollectibleType.COLLECTIBLE_MOMS_CONTACTS)
			player:RemoveCostume(dogToothConfig)
			player:RemoveCostume(momsContactsConfig)
			
			player:AddCacheFlags(CacheFlag.CACHE_SPEED)
			player:AddCacheFlags(CacheFlag.CACHE_DAMAGE)
			player:EvaluateItems()
		
		end
	
	end
	
end

--removes the costume and sets IsDog or IsWolf to false if they were a dog or wolf but are not pibbles or onyx anymore
--used to handle when the player transforms into another character, through revival or clicker or whatever
function mod:TryDeinitCharacter(player)

	local data = player:GetData()
	
	local isGhost = false
	if data.TrueCoop and data.TrueCoop.Save and data.TrueCoop.Save.IsGhost then
		isGhost = true
	end
	
	if data.IsDog and (player:GetPlayerType() ~= PlayerType.PLAYER_PIBBLES or isGhost) then
	
		data.IsDog = false
		player:TryRemoveNullCostume(NullItemID.ID_PIBBLES)
		
		player:AddCacheFlags(CacheFlag.CACHE_FIREDELAY)
		player:AddCacheFlags(CacheFlag.CACHE_DAMAGE)
		player:EvaluateItems()
	
	end
	
	if data.IsWolf and (player:GetPlayerType() ~= PlayerType.PLAYER_ONYX or isGhost) then
	
		data.IsWolf = false
		player:TryRemoveNullCostume(NullItemID.ID_ONYX)
		
		player:AddCacheFlags(CacheFlag.CACHE_SPEED)
		player:AddCacheFlags(CacheFlag.CACHE_DAMAGE)
		player:EvaluateItems()
	
	end
	
end

--guppy handling
function mod:TryInitGuppy(player)

	local data = player:GetData()
	
	local isGhost = false
	if data.TrueCoop and data.TrueCoop.Save and data.TrueCoop.Save.IsGhost then
		isGhost = true
	end
	
	if not isGhost then
	
		if (data.IsDog or data.IsWolf) and player:HasPlayerForm(PlayerForm.PLAYERFORM_GUPPY) and not data.IsAlsoCat then
		
			data.IsAlsoCat = true
			player:AddNullCostume(NullItemID.ID_GUPPY_PIBBLESONYX)
		
		end
	
	end
	
end
function mod:TryDeinitGuppy(player)

	local data = player:GetData()
	
	local isGhost = false
	if data.TrueCoop and data.TrueCoop.Save and data.TrueCoop.Save.IsGhost then
		isGhost = true
	end
	
	if (data.IsDog or data.IsWolf) and data.IsAlsoCat and not player:HasPlayerForm(PlayerForm.PLAYERFORM_GUPPY) then
	
		data.IsAlsoCat = false
		player:TryRemoveNullCostume(NullItemID.ID_GUPPY_PIBBLESONYX)
	
	end
	
end

--adds the costume back to the player if they just got an item or an item effect
--used in the case of D4, which removes null costumes. can we please have a way to force a costume to a player please nicalis
function mod:AddItemFix(player, itemID, checkEffect)

	local data = player:GetData()
	
	if data.IsDog or data.IsWolf then
	
		local hasItem
		if checkEffect then
			hasItem = player:GetEffects():HasCollectibleEffect(itemID)
		else
			hasItem = player:HasCollectible(itemID)
		end
	
		data.DogHadItem = data.DogHadItem or {}
		if hasItem and not data.DogHadItem[itemID] then
	
			if data.IsDog then
				player:AddNullCostume(NullItemID.ID_PIBBLES)
			elseif data.IsWolf then
				player:AddNullCostume(NullItemID.ID_ONYX)
			end
			
			if data.IsAlsoCat then
				player:AddNullCostume(NullItemID.ID_GUPPY_PIBBLESONYX)
			end
		
		end
	
		data.DogHadItem[itemID] = hasItem
	
	end
	
end

--adds the costume back to the player if they went down to a new floor while holding an item or item effect
--used in the case of missingno, which even though it triggers the d4 effect when picking it up, it doesnt do so when going to a new floor and still removes the costume. nicalis why do u gotta have two separate ways to reroll the player's items for one item. cant u just make it trigger d4 again?
function mod:NewFloorItemFix(player, itemID, checkEffect)

	local data = player:GetData()
	
	if data.IsDog or data.IsWolf then
	
		local hasItem
		if checkEffect then
			hasItem = player:GetEffects():HasCollectibleEffect(itemID)
		else
			hasItem = player:HasCollectible(itemID)
		end
	
		if hasItem then
	
			if data.IsDog then
				player:AddNullCostume(NullItemID.ID_PIBBLES)
			elseif data.IsWolf then
				player:AddNullCostume(NullItemID.ID_ONYX)
			end
			
			if data.IsAlsoCat then
				player:AddNullCostume(NullItemID.ID_GUPPY_PIBBLESONYX)
			end
		
		end
	
	end
	
end


-----------------
--PIBBLES CACHE--
-----------------

function mod:EvalFireDelayPibbles(player, cacheFlag)

	local data = player:GetData()

	if data.IsDog then
	
		player.MaxFireDelay = player.MaxFireDelay - 2
		
    end
	
end
mod:AddCallback(ModCallbacks.MC_EVALUATE_CACHE, mod.EvalFireDelayPibbles, CacheFlag.CACHE_FIREDELAY)

function mod:EvalDamagePibbles(player, cacheFlag)

	local data = player:GetData()

	if data.IsDog then
	
		player.Damage = (player.Damage - 0.75) * 0.9
		
    end
	
end
mod:AddCallback(ModCallbacks.MC_EVALUATE_CACHE, mod.EvalDamagePibbles, CacheFlag.CACHE_DAMAGE)


--------------
--ONYX CACHE--
--------------

function mod:EvalDamageOnyx(player, cacheFlag)

	local data = player:GetData()

	if data.IsWolf then
	
		player.Damage = (player.Damage - 0.2) * 1.3
		
    end
	
end
mod:AddCallback(ModCallbacks.MC_EVALUATE_CACHE, mod.EvalDamageOnyx, CacheFlag.CACHE_DAMAGE)

function mod:EvalSpeedOnyx(player, cacheFlag)

	local data = player:GetData()

	if data.IsWolf then
	
		player.MoveSpeed = player.MoveSpeed - 0.1
		
    end
	
end
mod:AddCallback(ModCallbacks.MC_EVALUATE_CACHE, mod.EvalSpeedOnyx, CacheFlag.CACHE_SPEED)


----------------------
--CALLBACK FUNCTIONS--
----------------------
--these functions get tied directly to callbacks

local newGame = false
function mod:PlayerInit(player)

	mod:PlayerUpdateAndInit(player)
	
	local game = Game()
	if game.TimeCounter == 0 or newGame then
	
		newGame = true
	
		local data = player:GetData()
		
		if data.IsDog then

			player:AddCollectible(CollectibleType.COLLECTIBLE_DOG_TOOTH, 0, true)
			player:AddCollectible(CollectibleType.COLLECTIBLE_PEE_BOARD, 6, false)
			
			local itemPool = game:GetItemPool()
			itemPool:RemoveCollectible(CollectibleType.COLLECTIBLE_DOG_TOOTH)
			itemPool:RemoveCollectible(CollectibleType.COLLECTIBLE_PEE_BOARD)
			
			--pibbles shouldnt have dog tooth's costume
			local itemConfig = Isaac.GetItemConfig()
			local dogToothConfig = itemConfig:GetCollectible(CollectibleType.COLLECTIBLE_DOG_TOOTH)
			player:RemoveCostume(dogToothConfig)
			
		end
		
		if data.IsWolf then

			player:AddCollectible(CollectibleType.COLLECTIBLE_DOG_TOOTH, 0, true)
			player:AddCollectible(CollectibleType.COLLECTIBLE_MOMS_CONTACTS, 0, true)
			
			--spawn items for onyx to choose from
			data.SpawnOnyxItems = true
			
			local itemPool = game:GetItemPool()
			itemPool:RemoveCollectible(CollectibleType.COLLECTIBLE_DOG_TOOTH)
			itemPool:RemoveCollectible(CollectibleType.COLLECTIBLE_MOMS_CONTACTS)
			itemPool:RemoveCollectible(CollectibleType.COLLECTIBLE_D6)
			itemPool:RemoveCollectible(CollectibleType.COLLECTIBLE_VOID)
			
			--onyx shouldnt have dog tooth's or mom's contacts' costumes
			local itemConfig = Isaac.GetItemConfig()
			local dogToothConfig = itemConfig:GetCollectible(CollectibleType.COLLECTIBLE_DOG_TOOTH)
			local momsContactsConfig = itemConfig:GetCollectible(CollectibleType.COLLECTIBLE_MOMS_CONTACTS)
			player:RemoveCostume(dogToothConfig)
			player:RemoveCostume(momsContactsConfig)
			
		end
	
	end
	
end
function mod:PlayerUpdateAndInit(player)

	mod:TryInitCharacter(player) --adds the costume to the player when they start the game
	mod:TryDeinitCharacter(player) --removes the costume from the player when they turn into a separate character
	
	mod:TryInitGuppy(player)
	mod:TryDeinitGuppy(player)
	
	mod:AddItemFix(player, CollectibleType.COLLECTIBLE_D4, true) --re-adds the costume when picking up missing no, using d4, using d100, and triggering the dice room
	
end
function mod:GameExit(menuExit)

	newGame = false
	
end

--more options hackery
local roomItems = {}
mod:AddCallback(ModCallbacks.MC_POST_RENDER, function()

	local game = Game()
	
	if not game:IsPaused() then
	
		local room = game:GetRoom()
		local level = game:GetLevel()
		local roomDesc = level:GetCurrentRoomDesc()
		
		for i=1, game:GetNumPlayers() do
		
			local player = Isaac.GetPlayer(i-1)
			local data = player:GetData()
			
			if data.SpawnOnyxItems then
			
				--spawn items for onyx to choose from
				local playerPos = player.Position
				
				local d6Pos = room:FindFreePickupSpawnPosition(playerPos + Vector(-50,-80), 0.0, true)
				local voidPos = room:FindFreePickupSpawnPosition(playerPos + Vector(50,-80), 0.0, true)
				
				local d6 = Isaac.Spawn(EntityType.ENTITY_PICKUP, PickupVariant.PICKUP_COLLECTIBLE, CollectibleType.COLLECTIBLE_D6, d6Pos, vecZero, nil)
				local void = Isaac.Spawn(EntityType.ENTITY_PICKUP, PickupVariant.PICKUP_COLLECTIBLE, CollectibleType.COLLECTIBLE_VOID, voidPos, vecZero, nil)
				
				roomItems[roomDesc.ListIndex] = roomItems[roomDesc.ListIndex] or {}
				
				local roomItemList = roomItems[roomDesc.ListIndex]
				roomItemList[#roomItemList+1] = {
					room:GetGridIndex(d6.Position),
					room:GetGridIndex(void.Position)
				}
				
				data.SpawnOnyxItems = false
				
			end
		
		end

		if roomItems[roomDesc.ListIndex] then
		
			local itemLists = roomItems[roomDesc.ListIndex]
			
			for h=1, #itemLists do
			
				local currentItemList = itemLists[h]
				
				if currentItemList then
					
					local roomItemsEntities = {}
					
					for _,entity in ipairs(Isaac.FindByType(EntityType.ENTITY_PICKUP, PickupVariant.PICKUP_COLLECTIBLE, -1, false, false)) do
					
						local gridIndex = room:GetGridIndex(entity.Position)
						for i=1, #currentItemList do
						
							if gridIndex == currentItemList[i] then
							
								roomItemsEntities[#roomItemsEntities+1] = entity:ToPickup()
								break
								
							end
							
						end
						
					end
					
					if #roomItemsEntities == #currentItemList then
						
						local removeOtherPickups = false
						
						for _,pickup in ipairs(roomItemsEntities) do
						
							if pickup.Touched or pickup.SubType == 0 then
						
								removeOtherPickups = true
								
							end
						
						end
						
						if removeOtherPickups then
						
							for _,pickup in ipairs(roomItemsEntities) do
							
								if not pickup.Touched and pickup.SubType ~= 0 then
							
									Isaac.Spawn(EntityType.ENTITY_EFFECT, EffectVariant.POOF01, 0, pickup.Position, vecZero, nil)
									pickup:Remove()
									
								end
							
							end
							
							itemLists[h] = false
							
						end
						
						break
					
					end
				
				end
				
			end
		
		end
	
	end
	
end)
mod:AddCallback(ModCallbacks.MC_POST_NEW_LEVEL, function()
	roomItems = {}
end)
mod:AddCallback(ModCallbacks.MC_PRE_PICKUP_COLLISION, function(_, pickup, collider)

	if pickup.Variant == PickupVariant.PICKUP_COLLECTIBLE and collider:ToPlayer() then
	
		local game = Game()
		local room = game:GetRoom()
		local level = game:GetLevel()
		local roomDesc = level:GetCurrentRoomDesc()
		
		if roomItems[roomDesc.ListIndex] then
		
			local itemLists = roomItems[roomDesc.ListIndex]
			
			for h=1, #itemLists do
			
				local currentItemList = itemLists[h]
				
				if currentItemList then
					
					local isOnyxItem = false
				
					local gridIndex = room:GetGridIndex(pickup.Position)
					for i=1, #currentItemList do
					
						if gridIndex == currentItemList[i] then
						
							isOnyxItem = true
							break
							
						end
						
					end
					
					if isOnyxItem then
					
						local colliderData = collider:GetData()
						
						if not colliderData.IsWolf then
							return false
						end
						
						break
						
					end
				
				end
			
			end
		
		end
		
	end
	
end)

mod:AddCallback(ModCallbacks.MC_POST_PLAYER_INIT, mod.PlayerInit, 0)
mod:AddCallback(ModCallbacks.MC_POST_PLAYER_UPDATE, mod.PlayerUpdateAndInit, 0)
mod:AddCallback(ModCallbacks.MC_PRE_GAME_EXIT, mod.GameExit)

function mod:NewLevel()

	local game = Game()
	for i=1, game:GetNumPlayers() do
	
		local player = Isaac.GetPlayer(i-1)
	
		mod:NewFloorItemFix(player, CollectibleType.COLLECTIBLE_MISSING_NO) --re-adds the costume when transitioning to a new floor while holding missing no
		
	end
	
end

mod:AddCallback(ModCallbacks.MC_POST_NEW_LEVEL, mod.NewLevel)


------------------
--ITEM FUNCTIONS--
------------------
--these functions are directly tied to items

--pee board
local pissColors = {
	Color(0.5,0.5,0.5,1,100,100,0),
	Color(0.5,0.5,0.5,1,150,150,0),
	Color(0.5,0.5,0.5,1,180,180,0),
	Color(0.5,0.5,0.5,1,200,200,0),
	Color(0.5,0.5,0.5,1,215,215,0),
	Color(0.5,0.5,0.5,1,230,230,0),
	Color(0.5,0.5,0.5,1,245,245,0),
	Color(0.5,0.5,0.5,1,255,255,0)
}
local pissActive = false
local pissUses = 0
function mod:UsePeeBoard(itemID, itemRNG)

	local game = Game()
	local room = game:GetRoom()
	
	pissActive = true
	pissUses = pissUses + 1
	
	local pissColor = pissColors[pissUses] or pissColors[#pissColors]
	room:SetFloorColor(pissColor)

	return true
	
end
mod:AddCallback(ModCallbacks.MC_USE_ITEM, mod.UsePeeBoard, CollectibleType.COLLECTIBLE_PEE_BOARD)

function mod:NpcUpdatePeeBoard(npc)

	if pissActive and npc:IsVulnerableEnemy() and not npc:IsFlying() and npc:IsFrame(6,0) then
		
		local basePissDamage = 2
		local currentPissDamage = 0
		for i=1, pissUses do
			currentPissDamage = currentPissDamage + (basePissDamage / i)
		end
		
		npc:TakeDamage(currentPissDamage, DamageFlag.DAMAGE_ACID, EntityRef(npc), 0)

	end
	
end
mod:AddCallback(ModCallbacks.MC_NPC_UPDATE, mod.NpcUpdatePeeBoard)

function mod:ResetPeeBoard()

	pissActive = false
	pissUses = 0
	
end
mod:AddCallback(ModCallbacks.MC_POST_NEW_ROOM, mod.ResetPeeBoard)

--femboy
function mod:NpcUpdateFemboy(npc)

	if npc:IsVulnerableEnemy() then
	
		local game = Game()
		local room = game:GetRoom()

		local data = npc:GetData()
		if not data.DidFemboyCharmCheck and room:GetFrameCount() <= 10 then

			for i=1, game:GetNumPlayers() do
			
				local player = Isaac.GetPlayer(i-1)
				
				if player.Variant == 0 and player:HasCollectible(CollectibleType.COLLECTIBLE_FEMBOY) then --non coop babies with femboy
				
					local charmed = false
					local numSocks = player:GetCollectibleNum(CollectibleType.COLLECTIBLE_FEMBOY)
					local charmRNG = player:GetCollectibleRNG(CollectibleType.COLLECTIBLE_FEMBOY)
					for j=1, numSocks do
					
						local charmChance = math.max(math.min(20 + (player.Luck * 6),100),10)
						if (charmRNG:RandomInt(9999)+1) * 0.01 <= charmChance then
						
							npc:AddCharmed(300)
							charmed = true
							break
							
						end
						
					end
					
					if charmed then
						break
					end
				
				end
				
			end
			
			data.DidFemboyCharmCheck = true

		end
		
	end
	
end
mod:AddCallback(ModCallbacks.MC_NPC_UPDATE, mod.NpcUpdateFemboy)

function mod:EvalSpeedFemboy(player, cacheFlag)

	if player:HasCollectible(CollectibleType.COLLECTIBLE_FEMBOY) then
	
		local numSocks = player:GetCollectibleNum(CollectibleType.COLLECTIBLE_FEMBOY)
		player.MoveSpeed = player.MoveSpeed + (0.2 * numSocks)
		
    end
	
end
mod:AddCallback(ModCallbacks.MC_EVALUATE_CACHE, mod.EvalSpeedFemboy, CacheFlag.CACHE_SPEED)

--onyx's gaze
local onyxsGazeTearColor = Color(1.25, 1.0, 0.15, 1.0, 15, 15, 0)
function mod:OnyxsGazeTearInit(tear)

	local data = tear:GetData()
	if not data.OnyxGazeTear and tear.TearFlags & TearFlags.TEAR_FREEZE ~= 0 and tear.Parent and tear.Parent:GetData().IsWolf then
	
		data.OnyxGazeTear = true
		
		tear.Color = onyxsGazeTearColor
		
	end
	
end
mod:AddCallback(ModCallbacks.MC_POST_TEAR_UPDATE, mod.OnyxsGazeTearInit)

--lycanthropy
function mod:EvalDamageLycanthropy(player, cacheFlag)

	if player:HasCollectible(CollectibleType.COLLECTIBLE_LYCANTHROPY) then
	
		local data = player:GetData()
		
		local playerIndex = 1
		if data.TrueCoop and data.TrueCoop.PlayerListIndex then
			playerIndex = data.TrueCoop.PlayerListIndex
		end

		local saveData = SaveHelper.LevelSave(mod)
		
		if saveData.LycanthropyActive[playerIndex] then
	
			local numFangs = player:GetCollectibleNum(CollectibleType.COLLECTIBLE_LYCANTHROPY)
			player.Damage = player.Damage + (3 * numFangs)
			
		end
		
    end
	
end
mod:AddCallback(ModCallbacks.MC_EVALUATE_CACHE, mod.EvalDamageLycanthropy, CacheFlag.CACHE_DAMAGE)

function mod:NewRoomLycanthropy()

	local game = Game()
	local room = game:GetRoom()
	local roomType = room:GetType()
	
	if roomType == RoomType.ROOM_SECRET or roomType == RoomType.ROOM_SUPERSECRET then
	
		local saveData = SaveHelper.LevelSave(mod)

		for i=1, game:GetNumPlayers() do
	
			local player = Isaac.GetPlayer(i-1)
			
			if player:HasCollectible(CollectibleType.COLLECTIBLE_LYCANTHROPY) then
			
				local data = player:GetData()
				
				local playerIndex = i
				if data.TrueCoop and data.TrueCoop.PlayerListIndex then
					playerIndex = data.TrueCoop.PlayerListIndex
				end
			
				if not saveData.LycanthropyActive[playerIndex] then
				
					saveData.LycanthropyActive[playerIndex] = true
					
					player:AddCacheFlags(CacheFlag.CACHE_DAMAGE)
					player:EvaluateItems()
					
					local sfx = SFXManager()
					sfx:Play(SoundEffect.SOUND_DOG_HOWELL, 1.0, 0, false, 1.0)
					
					if not data.IsDog and not data.IsWolf then
					
						player:AddNullCostume(NullItemID.ID_LYCANTHROPY)
					
					end
				
				end
			
			end
		
		end
	
	end
	
end
mod:AddCallback(ModCallbacks.MC_POST_NEW_ROOM, mod.NewRoomLycanthropy)

function mod:PostResetLevelSaveLycanthropy(modRef, saveData)

	local game = Game()
	for i=1, game:GetNumPlayers() do
	
		local player = Isaac.GetPlayer(i-1)
		
		local data = player:GetData()
		
		local playerIndex = i
		if data.TrueCoop and data.TrueCoop.PlayerListIndex then
			playerIndex = data.TrueCoop.PlayerListIndex
		end
		
		if saveData.LycanthropyActive[playerIndex] then
	
			player:TryRemoveNullCostume(NullItemID.ID_LYCANTHROPY)
			
			player:AddCacheFlags(CacheFlag.CACHE_DAMAGE)
			player:EvaluateItems()
		
		end
		
	end

end
mod:AddCustomCallback(CustomCallbacks.SH_PRE_RESET_LEVEL, mod.PostResetLevelSaveLycanthropy, mod.Name)


-------------
--TRUE COOP--
-------------

local function onTrueCoopInit()

	local pibblesTrueCoopFetus = Sprite()
	pibblesTrueCoopFetus:Load("gfx/truecoop/player_fetus/01_Isaac.anm2", true)
	pibblesTrueCoopFetus:ReplaceSpritesheet(3, "gfx/characters/peeboard/costumes/character_pibbles_fetus.png")
	pibblesTrueCoopFetus:LoadGraphics()
	pibblesTrueCoopFetus:Play("Idle", true)

	local pibblesTrueCoopData = {
		Name = "Pibbles",
		Type = PlayerType.PLAYER_ISAAC,
		SelectionGfx = "gfx/ui/portraits/peeboard/pibbles_coop.png",
		GhostCostume = NullItemID.ID_PIBBLES_GHOST,
		MaxHearts = 6,
		Hearts = 6,
		AllowHijacking = true,
		ActualType = PlayerType.PLAYER_PIBBLES,
		FetusSprite = pibblesTrueCoopFetus,
		BossPortrait = "gfx/ui/portraits/peeboard/pibbles_portrait.png",
		BossName = "gfx/ui/portraits/peeboard/pibbles_name.png",
		GhostPortrait = "gfx/ui/portraits/peeboard/pibbles_portrait_ghost.png",
		GhostName = "gfx/ui/portraits/peeboard/pibbles_name_ghost.png",
		OnStart = function(player)
		
			local sprite = player:GetSprite()
			sprite:ReplaceSpritesheet(1, "gfx/characters/peeboard/costumes/character_pibbles.png")
			sprite:ReplaceSpritesheet(4, "gfx/characters/peeboard/costumes/character_pibbles.png")
			sprite:ReplaceSpritesheet(12, "gfx/characters/peeboard/costumes/character_pibbles.png")
			sprite:LoadGraphics()
			
			mod:PlayerInit(player)
			
		end
	}

	InfinityTrueCoopInterface.AddCharacter(pibblesTrueCoopData)
	InfinityTrueCoopInterface.AddCharacterToWheel("Pibbles")
	InfinityTrueCoopInterface.AssociatePlayerTypeName(PlayerType.PLAYER_PIBBLES, "Pibbles")

	local onyxTrueCoopFetus = Sprite()
	onyxTrueCoopFetus:Load("gfx/truecoop/player_fetus/01_Isaac.anm2", true)
	onyxTrueCoopFetus:ReplaceSpritesheet(3, "gfx/characters/peeboard/costumes/character_onyx_fetus.png")
	onyxTrueCoopFetus:LoadGraphics()
	onyxTrueCoopFetus:Play("Idle", true)

	local onyxTrueCoopData = {
		Name = "Onyx",
		Type = PlayerType.PLAYER_ISAAC,
		SelectionGfx = "gfx/ui/portraits/peeboard/onyx_coop.png",
		GhostCostume = NullItemID.ID_ONYX_GHOST,
		BlackHearts = 4,
		AllowHijacking = true,
		ActualType = PlayerType.PLAYER_ONYX,
		FetusSprite = onyxTrueCoopFetus,
		BossPortrait = "gfx/ui/portraits/peeboard/onyx_portrait.png",
		BossName = "gfx/ui/portraits/peeboard/onyx_name.png",
		GhostPortrait = "gfx/ui/portraits/peeboard/onyx_portrait_ghost.png",
		GhostName = "gfx/ui/portraits/peeboard/onyx_name_ghost.png",
		OnStart = function(player)
		
			local sprite = player:GetSprite()
			sprite:ReplaceSpritesheet(1, "gfx/characters/peeboard/costumes/character_onyx.png")
			sprite:ReplaceSpritesheet(4, "gfx/characters/peeboard/costumes/character_onyx.png")
			sprite:ReplaceSpritesheet(12, "gfx/characters/peeboard/costumes/character_onyx.png")
			sprite:LoadGraphics()
			
			mod:PlayerInit(player)
			
		end
	}

	InfinityTrueCoopInterface.AddCharacter(onyxTrueCoopData)
	InfinityTrueCoopInterface.AddCharacterToWheel("Onyx")
	InfinityTrueCoopInterface.AssociatePlayerTypeName(PlayerType.PLAYER_ONYX, "Onyx")

end

if InfinityTrueCoopInterface then
	onTrueCoopInit()
else
	if not __infinityTrueCoop then
		__infinityTrueCoop = {}
	end

	__infinityTrueCoop[#__infinityTrueCoop + 1] = onTrueCoopInit
end
